/*------------------------------------------------
 * Proyecto: Anarit
 * Autor:    Pedro Reina <pedro@pedroreina.net>
 * Archivo:  AnaritActivity.java
 * Fecha:    X.27.7.2011
 * Licencia: GPLv2
 *----------------------------------------------*/

// El paquete de este programa
package net.pedroreina.anarit;

// Manejar la actividad
import android.app.Activity;
import android.os.Bundle;

// Manejar el menú
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;

// Usar diálogos de alerta
import android.app.Dialog;
import android.app.AlertDialog;
import android.app.AlertDialog.Builder;

// Convertir entre números y cadenas
import java.lang.Integer;
import java.lang.Long;

// Usar listas
import java.util.ArrayList;

// Usar el reloj de precisión
import android.os.SystemClock;

// Manejar textos con etiquetas HTML
import android.text.Html;
import android.text.Spanned;

// Lanzar la actividad de las preferencias
import android.content.Intent;

// Leer las preferencias
import android.preference.PreferenceManager;
import android.content.SharedPreferences;
import android.content.SharedPreferences.OnSharedPreferenceChangeListener;

// Elementos del GUI
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

// Vistas
import android.view.View;

// Mostrar toast
import android.widget.Toast;

//-----------------------------------------------
// La actividad principal
//-----------------------------------------------
public class AnaritActivity extends Activity
  implements OnSharedPreferenceChangeListener
  {
  // Declaración y carga del método nativo
  public native String Analiza
    (int d1, int d2, int d3, int d4, int d5, int d6, int meta);
  static { System.loadLibrary ("anarit"); }

  // Las constantes que definen el modo de pedir el problema
  public static final int MODO_NORMAL = 0;
  public static final int MODO_EXPERTO = 1;

  // Los textos de cada botón
  String Valor[] = {"0", "1", "2", "3", "4", "5", "6", "7", "8", "9",
                    "10", "25", "50", "75", "100" };

  AlertDialog About, License, Credits;
  Intent Config, Help;

  // Botones
  Button BotonAnalisis, BotonBorrar, BotonNuevo;
  Button[] BotonDato;

  // El lugar en el que editar los datos del problema
  EditText TextoDatos;

  // El lugar en el que ir escribiendo los datos del problema
  TextView TextoProblema;

  // El problema que estamos preparando (modos normal y experto)
  String PreparacionProblema, ProblemaExperto;

  // Los textos con los resultados
  TextView TextoObtenido, TextoOperaciones, TextoComplejidad, TextoTiempo;

  // Conjunto de preferencias
  SharedPreferences Pref;

  // Manejo del modo de pedir el problema
  int Modo;
  String ModoDefecto;

  // El dato que va a introducir el usuario
  int PosicionEntrada;

  // El estado de la actividad
  Bundle Estado;

  //----------------------------------------------
  // El sistema crea la actividad
  //----------------------------------------------
  @Override
  public void onCreate (Bundle EstadoSalvado)
    {
    AlertDialog.Builder Constructor;

    super.onCreate (EstadoSalvado);
    Estado = EstadoSalvado;

    BotonDato = new Button[15];

    // Averiguamos el modo por defecto
    ModoDefecto = getString (R.string.mode_default);

    // Preparamos el lector de preferencias
    Pref = PreferenceManager.getDefaultSharedPreferences (this);
    Pref.registerOnSharedPreferenceChangeListener (this);

    // Creamos el diálogo About
    Constructor = new AlertDialog.Builder(this);
    String Mensaje = "<b>" + getString (R.string.app_name) + "</b>";
      Mensaje += "<BR>" + getString (R.string.version);
      Mensaje += " " + getString (R.string.num_version);
      Mensaje += ", " + getString (R.string.fecha);
      Mensaje += "<BR>Pedro Reina";
      Mensaje += "<BR>http://pedroreina.net";
    Constructor.setTitle (getString (R.string.about));
    Constructor.setMessage (Html.fromHtml (Mensaje));
    Constructor.setPositiveButton (getString(R.string.OK), null);
    About = Constructor.create();

    // Creamos el diálogo License
    Constructor = new AlertDialog.Builder (this);
    Constructor.setTitle (getString (R.string.license));
    Constructor.setMessage (getText (R.string.text_license));
    Constructor.setPositiveButton (getString (R.string.OK), null);
    License = Constructor.create();

    // Creamos el diálogo Credits
    Constructor = new AlertDialog.Builder(this);
    Constructor.setTitle (getString (R.string.credits));
    Constructor.setMessage (getText (R.string.text_credits));
    Constructor.setPositiveButton (getString (R.string.OK), null);
    Credits = Constructor.create();

    // Preparamos la actividad Preferences
    Config = new Intent (getBaseContext(), AnaritPreferences.class);

    // Preparamos la actividad Help
    Help = new Intent (getBaseContext(), AnaritHelp.class);

    // Lanzamos la vista que corresponda
    PreparaVista();
    }

  //----------------------------------------------
  // Creación del menú de opciones
  //----------------------------------------------
  @Override
  public boolean onCreateOptionsMenu (Menu menu)
    {
    MenuInflater inflater = getMenuInflater();
    inflater.inflate (R.menu.options, menu);
    return true;
    }

  //----------------------------------------------
  // Manejo de las contestaciones del menú de opciones
  //----------------------------------------------
  @Override
  public boolean onOptionsItemSelected (MenuItem item)
    {
    // Actuamos segun elija el usuario
    switch ( item.getItemId() )
      {
      case R.id.config:
        startActivity (Config);
        return true;

      case R.id.help:
        startActivity (Help);
        return true;

      case R.id.license:
        License.show();
        return true;

      case R.id.credits:
        Credits.show();
        return true;

      case R.id.about:
        About.show();
        return true;

      case R.id.quit:
        // Cerramos el programa
        this.finish();
        System.runFinalizersOnExit (true);
        System.exit (0);
        return true;

      default:
        return super.onOptionsItemSelected (item);
      }
    }

  //----------------------------------------------
  // Cuando arranca la vista de la actividad
  //----------------------------------------------
  @Override
  public void onStart()
    {
    super.onStart();

    if ( Estado == null )
      {
      // Valores iniciales
      ProblemaExperto = "";
      PreparacionProblema = "";
      PosicionEntrada = 0;
      }
    else
      { LeeEstado(); }

    PreparaVista();
    }

  //----------------------------------------------
  // Si el sistema avisa de que vamos a salir de la pantalla
  //----------------------------------------------
  @Override
  protected void onSaveInstanceState (Bundle Estado)
    {
    super.onSaveInstanceState (Estado);
    EscribeEstado (Estado);
    }

  //----------------------------------------------
  // Si el sistema avisa de que nos van a pausar
  //----------------------------------------------
  @Override
  protected void onPause()
    {
    super.onPause();
    if ( Estado == null )
      { Estado = new Bundle(); }
    EscribeEstado (Estado);
    }

  //----------------------------------------------
  // Si el sistema avisa de que vamos a volver
  //----------------------------------------------
  @Override
  protected void onResume()
    {
    super.onResume();
    if ( Estado != null )
      {
      LeeEstado();
      PreparaVista();
      }
    }

  //----------------------------------------------
  // Leer las variables del estado
  //----------------------------------------------
  public void LeeEstado()
    {
    PosicionEntrada = Estado.getInt ("posicion");
    ProblemaExperto = Estado.getString ("experto");
    PreparacionProblema = Estado.getString ("normal");
    }

  //----------------------------------------------
  // Escribir las variables del estado
  //----------------------------------------------
  public void EscribeEstado (Bundle Estado)
    {
    Estado.putInt ("posicion", PosicionEntrada);

    if ( Modo == MODO_EXPERTO )
      {
      Estado.putString ("experto", TextoDatos.getText().toString());
      Estado.putString ("normal", PreparacionProblema);
      }

    if ( Modo == MODO_NORMAL )
      {
      Estado.putString ("experto", ProblemaExperto);
      Estado.putString ("normal", PreparacionProblema);
      }
    }

  //----------------------------------------------
  // Si se cambia algún valor de las preferencias
  //----------------------------------------------
  @Override
  public void onSharedPreferenceChanged (SharedPreferences Pref, String Key)
    { PreparaVista(); }

  //----------------------------------------------
  // Preparar el modo de pedir el problema
  //----------------------------------------------
  public void PreparaVista()
    {
    String ModoTexto;

    // Leemos el modo de pedir el problema
    ModoTexto = Pref.getString ("mode", ModoDefecto);

    if ( ModoTexto.equals ("normal") )
      {
      Modo = MODO_NORMAL;
      setContentView (R.layout.normal);
      PreparaVistaNormal();
      PreparaVistaComun();
      TextoProblema.setText (PreparacionProblema);
      PreparaBotonera();
      }
    else
      {
      Modo = MODO_EXPERTO;
      setContentView (R.layout.experto);
      PreparaVistaExperto();
      PreparaVistaComun();
      TextoDatos.setText (ProblemaExperto);
      }
    }

  //----------------------------------------------
  // Preparar el GUI del modo normal
  //----------------------------------------------
  public void PreparaVistaNormal()
    {
    // Preparamos el texto donde prepararemos el problema
    TextoProblema = (TextView) findViewById (R.id.problema);

    // Buscamos el botón de borrar
    BotonBorrar = (Button) findViewById (R.id.borrar);

    // Le asignamos su función al pulsar
    BotonBorrar.setOnClickListener (new View.OnClickListener()
      {
      public void onClick (View Vista)
        { BorraUnDato(); }
      });

    // Le asignamos su función con pulsación larga
    BotonBorrar.setOnLongClickListener (new View.OnLongClickListener()
      {
      public boolean onLongClick (View Vista)
        {
        BorraDatos();
        return true;
        }
      });

    // Buscamos el botón de nuevo problema
    BotonNuevo = (Button) findViewById (R.id.nuevo);

    // Le asignamos su función
    BotonNuevo.setOnClickListener (new View.OnClickListener()
      {
      public void onClick (View Vista)
        { NuevoProblema(); }
      });

    // Buscamos los botones para introducir datos
    BotonDato[0] = (Button) findViewById (R.id.n0);
    BotonDato[1] = (Button) findViewById (R.id.n1);
    BotonDato[2] = (Button) findViewById (R.id.n2);
    BotonDato[3] = (Button) findViewById (R.id.n3);
    BotonDato[4] = (Button) findViewById (R.id.n4);
    BotonDato[5] = (Button) findViewById (R.id.n5);
    BotonDato[6] = (Button) findViewById (R.id.n6);
    BotonDato[7] = (Button) findViewById (R.id.n7);
    BotonDato[8] = (Button) findViewById (R.id.n8);
    BotonDato[9] = (Button) findViewById (R.id.n9);
    BotonDato[10] = (Button) findViewById (R.id.n10);
    BotonDato[11] = (Button) findViewById (R.id.n25);
    BotonDato[12] = (Button) findViewById (R.id.n50);
    BotonDato[13] = (Button) findViewById (R.id.n75);
    BotonDato[14] = (Button) findViewById (R.id.n100);

    // Asignamos a los botones su función
    for ( int i=0 ; i<15 ; i++ )
      {
      BotonDato[i].setOnClickListener (new View.OnClickListener()
        {
        public void onClick (View Vista)
          { BotonDatoPulsado (Vista); }
        });
      }
    }

  //----------------------------------------------
  // Preparar el GUI del modo experto
  //----------------------------------------------
  public void PreparaVistaExperto()
    { TextoDatos = (EditText) findViewById (R.id.datos); }

  //----------------------------------------------
  // Preparar las partes del GUI comunes a los modos
  //----------------------------------------------
  public void PreparaVistaComun()
    {
    // Buscamos el botón de analizar
    BotonAnalisis = (Button) findViewById (R.id.analiza);

    // Asignamos al botón su función
    BotonAnalisis.setOnClickListener (new View.OnClickListener()
      {
      public void onClick (View Vista)
        { AnalizaJava(); }
      });

    // Buscamos las vistas de los textos del GUI
    TextoObtenido = (TextView) findViewById (R.id.obtenido);
    TextoOperaciones = (TextView) findViewById (R.id.operaciones);
    TextoComplejidad = (TextView) findViewById (R.id.complejidad);
    TextoTiempo = (TextView) findViewById (R.id.tiempo);
    }

  //----------------------------------------------
  // Preparamos qué botones verá el usuario
  //----------------------------------------------
  public void PreparaBotonera()
    {
    // Si el problema está completo
    if ( PosicionEntrada == 9 )
      {
      // No se muestra ningún botón de datos
      for ( int i=0 ; i<15 ; i++ )
        {
        BotonDato[i].setText ("");
        BotonDato[i].setClickable (false);
        }
      // Se activa el botón de análisis
      BotonAnalisis.setVisibility (View.VISIBLE);
      }

    // Si el problema no está completo
    else
      {
      // Algunos botones se muestran y otros no
      for ( int i=0 ; i<15 ; i++ )
        {
        BotonDato[i].setText ((Valor[i]));
        BotonDato[i].setClickable (true);
        }
      // El botón de 0 solo se puede mostrar en los dos últimos dígitos
      if ( PosicionEntrada < 7 )
        {
        BotonDato[0].setText ("");
        BotonDato[0].setClickable (false);
        }
      // El botón de Borrar no se muestra al comenzar
      if ( PosicionEntrada == 0 )
        {
        BotonBorrar.setText ("");
        BotonBorrar.setClickable (false);
        }
      else
        {
        BotonBorrar.setText (getString (R.string.borrar));
        BotonBorrar.setClickable (true);
        }
      // Los botones del 10 al 100 solo se pueden mostrar en las
      // seis primeras posiciones
      if ( PosicionEntrada > 5 )
        {
        for ( int i=10 ; i<15 ; i++ )
          {
          BotonDato[i].setText ("");
          BotonDato[i].setClickable (false);
          }
        }
      // Se desactiva el botón de análisis
      BotonAnalisis.setVisibility (View.GONE);
      }
    }

  //----------------------------------------------
  // Cuando se pulsa un botón de introducción de datos
  //----------------------------------------------
  public void BotonDatoPulsado (View Boton)
    {
    String NuevoDato = "";

    // Buscamos qué botón nos mandan
    for ( int i=0 ; i<15 ; i++ )
      {
      // Si lo encontramos
      if ( BotonDato[i] == (Button) Boton )
        { NuevoDato += Valor[i]; }
      }

    // Actualizamos el problema
    if ( PreparacionProblema.length() > 0 & PosicionEntrada < 7 )
      { PreparacionProblema += " "; };
    PreparacionProblema += NuevoDato;

    // Avanzamos la posición
    PosicionEntrada++;

    // Si hemos llegado al objetivo, lo avisamos
    if ( PosicionEntrada == 6 )
      { PreparacionProblema += " =>"; }

    // Anotamos cómo va el problema
    TextoProblema.setText (PreparacionProblema);

    // Repasamos los botones
    PreparaBotonera();
    }

  //----------------------------------------------
  // Borramos los datos introducidos en el problema hasta el momento
  //----------------------------------------------
  public void BorraDatos()
    {
    PreparacionProblema = "";
    TextoProblema.setText (PreparacionProblema);
    PosicionEntrada = 0;
    PreparaBotonera();
    }

  //----------------------------------------------
  // Borramos el último dato introducido
  //----------------------------------------------
  public void BorraUnDato()
    {
    int Posicion;

    // Solo hay que hacer algo cuando hay datos
    if ( PosicionEntrada > 0 )
      {
      // Si solo hay un dato, lo borramos
      if ( PosicionEntrada == 1 )
        { PreparacionProblema = ""; }

      // Si estamos introduciendo los datos de partida
      else if ( PosicionEntrada < 6 )
        {
        // Buscamos el último espacio
        Posicion = PreparacionProblema.lastIndexOf (" ");
        // Nos quedamos con la primera parte
        PreparacionProblema = PreparacionProblema.substring (0, Posicion);
        }

      // Si estamos esperando el comienzo del número objetivo
      else if ( PosicionEntrada == 6 )
        {
        // Quitamos la flecha
        Posicion = PreparacionProblema.length();
        PreparacionProblema = PreparacionProblema.substring (0, Posicion-3);
        // Buscamos el último espacio
        Posicion = PreparacionProblema.lastIndexOf (" ");
        // Nos quedamos con la primera parte
        PreparacionProblema = PreparacionProblema.substring (0, Posicion);
        }

      // Si se ha escrito un dígito del número objetivo
      else if ( PosicionEntrada == 7 )
        {
        // Quitamos el dígito y el espacio anterior
        Posicion = PreparacionProblema.length();
        PreparacionProblema = PreparacionProblema.substring (0, Posicion-2);
        }

      // Si se ha escrito más de un dígito del número objetivo
      else
        {
        // Quitamos el último dígito
        Posicion = PreparacionProblema.length();
        PreparacionProblema = PreparacionProblema.substring (0, Posicion-1);
        }

      // Retrasamos la posición
      PosicionEntrada--;

      // Anotamos cómo va el problema
      TextoProblema.setText (PreparacionProblema);

      // Repasamos los botones
      PreparaBotonera();
      }
    }

  //----------------------------------------------
  // Preparamos el modo normal para un nuevo problema
  //----------------------------------------------
  public void NuevoProblema()
    {
    for ( int i=0 ; i<15 ; i++ )
      { BotonDato[i].setVisibility (View.VISIBLE); }
    BotonBorrar.setVisibility (View.VISIBLE);
    BotonNuevo.setVisibility (View.GONE);
    TextoObtenido.setText ("");
    TextoOperaciones.setText ("");
    TextoComplejidad.setText ("");
    TextoTiempo.setText ("");
    BorraDatos();
    }

  //----------------------------------------------
  // Preparamos en el modo normal todas las vistas
  //----------------------------------------------
  public void PreparaParaDarSolucion()
    {
    for ( int i=0 ; i<15 ; i++ )
      {
      BotonDato[i].setText ("");
      BotonDato[i].setVisibility (View.GONE);
      }
    BotonBorrar.setVisibility (View.GONE);
    BotonAnalisis.setVisibility (View.GONE);
    BotonNuevo.setVisibility (View.VISIBLE);
    }

  //----------------------------------------------
  // Analizar el problema
  //----------------------------------------------
  public void AnalizaJava()
    {
    String Problema="", Datos[], ResultadoCrudo, Resultado[], Operaciones;
    String NodosVistos, NodosTotal, Obtenido;
    int d1, d2, d3, d4, d5, d6, meta;
    long Antes, Ahora, Tiempo;

    // Obtenemos el problema
    if ( Modo == MODO_EXPERTO )
      { Problema = TextoDatos.getText().toString(); }
    if ( Modo == MODO_NORMAL )
      {
      Problema = TextoProblema.getText().toString();
      Problema = Problema.replace (" =>", "");
      }

    // Descomponemos el problema en los siete números
    Datos = Problema.split (" ");

    if ( Datos.length == 7 )
      {
      // Los convertimos a enteros
      d1 = Integer.parseInt (Datos[0]);
      d2 = Integer.parseInt (Datos[1]);
      d3 = Integer.parseInt (Datos[2]);
      d4 = Integer.parseInt (Datos[3]);
      d5 = Integer.parseInt (Datos[4]);
      d6 = Integer.parseInt (Datos[5]);
      meta  = Integer.parseInt (Datos[6]);

      // Llamamos al código nativo en C para que analice el problema
      BotonAnalisis.setText (getString (R.string.analizando));
      Antes = SystemClock.uptimeMillis();
      ResultadoCrudo = Analiza (d1, d2, d3, d4, d5, d6, meta);
      Ahora = SystemClock.uptimeMillis();
      Tiempo = Ahora - Antes;

      // Descomponemos el resultado obtenido
      Resultado = ResultadoCrudo.split (";");
      NodosVistos = Resultado[0];
      NodosTotal = Resultado[1];
      Obtenido = Resultado[2];
      Operaciones = "\n" + getString (R.string.operaciones) +
                    ":\n" + Resultado[3];
      Operaciones = Operaciones.replace ("*", "×");
      Operaciones = Operaciones.replace ("-", "–");
      Operaciones = Operaciones.replace ("/", "÷");
      Operaciones = Operaciones.replace (",", "\n");

      // Quizá el número obtenido sea el exacto
      if ( Obtenido.equals (Datos[6]) )
        {
        Obtenido = getString (R.string.obtenido) +
                   ": " + Obtenido + " " + getString (R.string.exacto);
        }
      else
        { Obtenido = getString (R.string.obtenido) + ": " + Obtenido; }

      // Anotamos el resultado
      if ( Modo == MODO_NORMAL )
        { PreparaParaDarSolucion(); }
      TextoObtenido.setText (Obtenido);
      TextoOperaciones.setText (Operaciones);
      TextoComplejidad.setText (getString (R.string.complejidad) +
                                ": " + NodosVistos + "/" + NodosTotal);
      TextoTiempo.setText (getString (R.string.tiempo) + ": " +
                           Long.toString (Tiempo) + " ms");
      BotonAnalisis.setText (getString (R.string.analiza));
      }
    else
      {
      // Informamos al usuario de que los datos son incorrectos
      MuestraToast (getString (R.string.datosmalos));
      }
    }

  //----------------------------------------------
  // Función que muestra un toast
  //----------------------------------------------
  private void MuestraToast (CharSequence Mensaje)
    {
    Toast toast;
    int Duracion = Toast.LENGTH_SHORT;
    toast = Toast.makeText (getApplicationContext(), Mensaje, Duracion);
    toast.show();
    }
  }
